"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processBulkResults = exports.processShopifyImages = void 0;
const gatsby_source_filesystem_1 = require("gatsby-source-filesystem");
const shopify_types_1 = require("./shopify-types");
const helpers_1 = require("./helpers");
async function processShopifyImages({ actions: { createNode }, createNodeId, cache, store, reporter, }, node) {
    const type = (0, helpers_1.parseShopifyId)(node.shopifyId)[1];
    const imageFields = shopify_types_1.shopifyTypes[type].imageFields;
    if (imageFields) {
        for (const fieldPath of imageFields) {
            const image = fieldPath
                .split(`.`)
                .reduce((acc, value) => acc[value], node);
            if (image && (0, helpers_1.parseImageExtension)(image.originalSrc) !== `gif`) {
                const fileNode = await (0, gatsby_source_filesystem_1.createRemoteFileNode)({
                    url: image.originalSrc,
                    cache,
                    createNode,
                    createNodeId,
                    parentNodeId: node.id,
                    store,
                    reporter,
                });
                image.localFile___NODE = fileNode.id;
            }
        }
    }
}
exports.processShopifyImages = processShopifyImages;
async function processBulkResults(gatsbyApi, pluginOptions, results) {
    let nodeCount = 0;
    const promises = [];
    const children = {};
    for (let i = results.length - 1; i >= 0; i -= 1) {
        const result = Object.assign({}, results[i]);
        const resultIsNode = result.id && JSON.stringify(Object.keys(result)) !== `["id","__parentId"]`;
        if (resultIsNode) {
            const type = (0, helpers_1.parseShopifyId)(result.id)[1];
            const imageFields = shopify_types_1.shopifyTypes[type].imageFields;
            const referenceFields = shopify_types_1.shopifyTypes[type].referenceFields;
            // Only increment for parent nodes
            if (!result.__parentId)
                nodeCount++;
            // Attach reference fields
            if (referenceFields) {
                for (const referenceField of referenceFields) {
                    result[referenceField] = [];
                }
            }
            // Attach children references / objects
            if (children[result.id]) {
                for (const child of children[result.id]) {
                    const childType = typeof child === `string`
                        ? (0, helpers_1.parseShopifyId)(child)[1]
                        : child.__typename;
                    const field = shopify_types_1.shopifyTypes[childType].key;
                    if (field) {
                        result[field] = [
                            typeof child === `string`
                                ? (0, helpers_1.createNodeId)(child, gatsbyApi, pluginOptions)
                                : child,
                            ...(result[field] || []),
                        ];
                    }
                    else {
                        throw new Error(`There is no shopifyType for child type: ${childType}`);
                    }
                }
            }
            if (!(0, helpers_1.isShopifyId)(result.id)) {
                throw new Error(`Expected an ID in the format gid://shopify/<typename>/<id>`);
            }
            const node = Object.assign(Object.assign({}, result), { id: (0, helpers_1.createNodeId)(result.id, gatsbyApi, pluginOptions), shopifyId: result.id, internal: {
                    type: `${pluginOptions.typePrefix || ``}Shopify${type}`,
                    contentDigest: gatsbyApi.createContentDigest(result),
                } });
            if (pluginOptions.downloadImages && imageFields) {
                promises.push(processShopifyImages(gatsbyApi, node).then(() => gatsbyApi.actions.createNode(node)));
            }
            else {
                gatsbyApi.actions.createNode(node);
            }
        }
        // If the object is a child store the reference
        if (result.__parentId) {
            children[result.__parentId] = [
                ...(children[result.__parentId] || []),
                result.id || result,
            ];
        }
    }
    await Promise.all(promises);
    return nodeCount;
}
exports.processBulkResults = processBulkResults;
//# sourceMappingURL=process-bulk-results.js.map