"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sourceNodes = void 0;
const events_1 = require("./events");
const shopify_types_1 = require("./shopify-types");
const create_operations_1 = require("./create-operations");
const source_from_operation_1 = require("./source-from-operation");
const helpers_1 = require("./helpers");
const deletionCounter = {};
/**
 * validateNodes - Recursive function that returns an array of node ids that are validated
 * @param gatsbyApi - Gatsby Helpers
 * @param pluginOptions - Plugin Options Object
 * @param nodeMap - Map Object of all nodes that haven't been deleted
 * @param nodes - Array of nodes to validate
 *
 * Note: This function is designed to receive all top-level nodes on the first pass
 * and then call itself recuresively afterwards for each nested layer of coupled nodes
 */
function validateNodes(gatsbyApi, pluginOptions, nodeMap, nodes, prevalidatedNodeIds) {
    let coupledNodeIds = [];
    let validatedNodeIds = [];
    for (const node of nodes) {
        if (!validatedNodeIds.includes(node.id) &&
            !(prevalidatedNodeIds === null || prevalidatedNodeIds === void 0 ? void 0 : prevalidatedNodeIds.includes(node.id))) {
            validatedNodeIds.push(node.id);
            const type = (0, helpers_1.parseShopifyId)(node.shopifyId)[1];
            const coupledNodeFields = shopify_types_1.shopifyTypes[type].coupledNodeFields;
            if (coupledNodeFields) {
                for (const field of coupledNodeFields) {
                    coupledNodeIds = coupledNodeIds.concat(node[field]);
                }
            }
        }
    }
    if (coupledNodeIds.length > 0) {
        const coupledNodes = [...new Set(coupledNodeIds)].map(id => nodeMap[id]);
        validatedNodeIds = validatedNodeIds.concat(validateNodes(gatsbyApi, pluginOptions, nodeMap, coupledNodes, validatedNodeIds));
    }
    return [...new Set(validatedNodeIds)];
}
function reportNodeDeletion(gatsbyApi, node) {
    const type = (0, helpers_1.parseShopifyId)(node.shopifyId)[1];
    deletionCounter[type] = (deletionCounter[type] || 0) + 1;
    const identifier = node.shopifyId
        ? `shopify ID: ${node.shopifyId}`
        : `internal ID: ${node.id}`;
    gatsbyApi.reporter.info(`Deleting ${type} node with ${identifier}`);
}
function reportDeletionSummary(gatsbyApi) {
    gatsbyApi.reporter.info(`===== SHOPIFY NODE DELETION SUMMARY =====`);
    for (const [key, value] of Object.entries(deletionCounter)) {
        gatsbyApi.reporter.info(`${key}: ${value}`);
    }
}
async function sourceNodes(gatsbyApi, pluginOptions) {
    const { typePrefix = ``, shopifyConnections: connections = [] } = pluginOptions;
    /* There is the potential for sync issues due to this. We use the lastBuildTime both
     * to query for changed nodes from Shopify, and to determine which nodes have been deleted.
     * If we do it at the beginning, there is the potential for us to reload the same changed
     * nodes in multiple subsequent builds. This is less of a concern that if we put it at the
     * end of the build, which would have the potential of not catching top-level nodes deleted
     * in Shopify during the build process. A clean build would be required after that in order
     * to remove the node as it wouldn't be removed in subsquent builds.
     */
    const lastBuildTime = (0, helpers_1.getLastBuildTime)(gatsbyApi, pluginOptions);
    (0, helpers_1.setLastBuildTime)(gatsbyApi, pluginOptions);
    const { productsOperation, productVariantsOperation, ordersOperation, collectionsOperation, locationsOperation, finishLastOperation, completedOperation, cancelOperationInProgress, } = (0, create_operations_1.createOperations)(gatsbyApi, pluginOptions, lastBuildTime);
    const sourceFromOperation = (0, source_from_operation_1.makeSourceFromOperation)(finishLastOperation, completedOperation, cancelOperationInProgress, gatsbyApi, pluginOptions, lastBuildTime);
    const operations = [productsOperation, productVariantsOperation];
    if (connections.includes(`orders`)) {
        operations.push(ordersOperation);
    }
    if (connections.includes(`collections`)) {
        operations.push(collectionsOperation);
    }
    if (connections.includes(`locations`)) {
        operations.push(locationsOperation);
    }
    for (const op of operations) {
        await sourceFromOperation(op);
    }
    if (lastBuildTime) {
        const nodeMap = Object.keys(shopify_types_1.shopifyTypes)
            .map(type => gatsbyApi.getNodesByType(`${typePrefix}Shopify${type}`))
            .reduce((acc, value) => acc.concat(value), [])
            .reduce((acc, value) => {
            return Object.assign(Object.assign({}, acc), { [value.id]: value });
        }, {});
        const { fetchDestroyEventsSince } = (0, events_1.eventsApi)(pluginOptions);
        const destroyEvents = await fetchDestroyEventsSince(lastBuildTime);
        destroyEvents.forEach(event => {
            const shopifyId = `gid://shopify/${event.subject_type}/${event.subject_id}`;
            const id = (0, helpers_1.createNodeId)(shopifyId, gatsbyApi, pluginOptions);
            reportNodeDeletion(gatsbyApi, nodeMap[id]);
            delete nodeMap[id];
        });
        const topLevelNodes = Object.values(nodeMap).filter(node => {
            const type = (0, helpers_1.parseShopifyId)(node.shopifyId)[1];
            return !shopify_types_1.shopifyTypes[type].coupled;
        });
        const validatedNodeIds = validateNodes(gatsbyApi, pluginOptions, nodeMap, topLevelNodes);
        for (const node of Object.values(nodeMap)) {
            if (validatedNodeIds.includes(node.id)) {
                gatsbyApi.actions.touchNode(node);
            }
            else {
                reportNodeDeletion(gatsbyApi, node);
            }
        }
        if (Object.values(nodeMap).length > validatedNodeIds.length) {
            reportDeletionSummary(gatsbyApi);
        }
    }
}
exports.sourceNodes = sourceNodes;
//# sourceMappingURL=source-nodes.js.map